import re
from math_operations import MathOperations  # Import math operations handler
from name_handler import NameHandler  # Import name handler

class HungarianChatbot:
    def __init__(self):
        self.math_operations = MathOperations()
        self.name_handler = NameHandler()
        self.topics = {
            
            'math': self.is_math_request,
            'name': self.is_name_introduction,
        }

    def get_response(self, message):
        print(f"[DEBUG] <BEJÖVŐ ÜZENET> {message}")  # Bejövő üzenet debug
        tokens = self.tokenize(message)
        print(f"[DEBUG] <TOKEN> {tokens}")

        return self.default_response(tokens)

    def tokenize(self, message):
        tokens = re.findall(r'\w+|[-+]?\d*\.?\d+|[+-/*]', message)
        return [token.lower() for token in tokens]  # Minden token kisbetűs

    def default_response(self, tokens):
        print(f"[DEBUG] <TOKENOK KIÉRTÉKELÉSE> {tokens}")  # Tokenok kiértékelése debug
        priority_topics = ['math', 'name']  # A prioritás rendje
        for topic in priority_topics:
            if self.topics[topic](tokens):
                print(f"[DEBUG] <TÉMA KIVÁLASZTÁS> {topic}")  # Téma kiválasztás debug
                handler = self.get_handler(topic)
                return handler(tokens)
        print(f"[DEBUG] <ERROR> Nincs megfelelő válasz.")  # Hiba debug
        return "Sajnálom, de nem tudok választ adni."

    def get_handler(self, topic):
        handlers = {
            'math': lambda tokens: self.math_operations.handle_math_request(tokens),
            'name': lambda tokens: self.name_handler.handle_name_introduction(tokens),
        }
        return handlers.get(topic, lambda x: "Sajnálom, de nem tudok választ adni.")

    def is_math_request(self, tokens):
        math_patterns = [
            r'\b(összead|adj össze|add össze|összege|összeadni|kivonás|mínusz|vond ki|kivonni|szorzás|szorozd meg|szorozni|osztás|oszd el|osztani)\b.*\b(\d+|kettő|három|négy|öt|hat|hét|nyolc|kilenc|tíz)\b',
            r'\b(mennyi|összesít|számlál|mérd|számítsd)\b',
            r'\b(egyenlő|fokozat|gyök|törtszám|egyszerűsíts|páros|páratlan|összeg|átlag|különbség|szorzat|osztó)\b',
            r'(\d+\s*[\+\-\*/]\s*\d+)'
        ]

        joined_tokens = ' '.join(tokens)
        return any(re.search(pattern, joined_tokens, re.IGNORECASE) for pattern in math_patterns)

    def is_name_introduction(self, tokens):
        name_patterns = [
            r'(\w+) vagyok',
            r'engem hívnak (\w+-?nak|\w+-?nek)', 
            r'az én nevem (\w+-?nak|\w+-?nek)', 
            r'(\w+-?nak|\w+-?nek) hívnak',
            r'(\w+-?nak|\w+-?nek) vagyok',
            r'a nevem (\w+-?nak|\w+-?nek)',
            r'úgy hívnak, hogy (\w+-?nak|\w+-?nek)',
            r'csak hívj (\w+-?nak|\w+-?nek)',
            r'így hívnak: (\w+-?nak|\w+-?nek)',
            r'hívnak (\w+-?nak|\w+-?nek)',
            r'engem (\w+-?nak|\w+-?nek) hívnak',
            r'nevezz (\w+-?nak|\w+-?nek)',
            r'(\w+-?nak|\w+-?nek) vagyok én',
            r'így hívnak engem, (\w+-?nak|\w+-?nek)',
            r'az én nevem, hogy (\w+-?nak|\w+-?nek)',
            r'hívnak engem (\w+-?nak|\w+-?nek)'
        ]

        joined_tokens = ' '.join(tokens)
        return any(re.search(pattern, joined_tokens, re.IGNORECASE) for pattern in name_patterns)
